@@ -2,9 +2,9 @@ from ground.base import Orientation
 from ground.hints import Point
 from reprit.base import generate_repr
 
-from .abcs import Node
 from .edge import Edge
 from .location import Location
+from .node import Node
 from .trapezoid import Trapezoid
 
 
