@@ -1,9 +1,7 @@
-from .core.trapezoidal.abcs import Graph
-from .core.trapezoidal.graph import to_graph_cls
+from .core.trapezoidal.graph import Graph
 from .core.trapezoidal.hints import Shuffler
 from .core.trapezoidal.location import Location
 
 Graph = Graph
 Location = Location
 Shuffler = Shuffler
-to_graph_cls = to_graph_cls
